import pandas as pd

# grupisanje i agregiranje

# ucitavamo podatke
# indeks mozemo da postavimo direktno prilikom ucitavanja podataka
# logicno je da id odgovora bude indeks
data = pd.read_csv("../Data/survey_results_public.csv", index_col="ResponseId")
data_schema = pd.read_csv("../Data/survey_results_schema.csv", index_col="qname")

# postavljamo granice
pd.set_option("display.max_columns", 114)
pd.set_option("display.max_rows", 114)


# agregiranje je kombinovanje vise podataka u jedan rezultat
# na primer, svima poznate agregatne funkcije su:
# 1. mean() - srednja vrednost
# 2. median() - medijana
# 3. mode() - najfrekventnija vrednost u skupu
# sve ove funkcije su agregatne, jer uzimaju veliki broj ulaznih podataka
# i transformisu ih u jedan broj

# na primer, zelimo da saznamo koja je prosecna plata developera koji 
# su ucestvovali u anketi

# posmatramo kolonu ConvertedCompYearly
print(data["ConvertedCompYearly"].head(15))
# primetimo da u veliki broj ispitanika nije odgovorio na ovo pitanje, 
# tj. imamo nedostajuce vrednosti (NaN)

# da bismo dobijali medijalnu zaradu, mozemo da primenimo median() metod
# median() ignorise NaN vrednosti
print("Medijalna zarada: ", data["ConvertedCompYearly"].median())
# mean() takodje ignorise nedostajuce vrednosti
# medijana je bolji opis podataka u odnosu na prosek. prosek je isuvise
# osetljiv na autlajere, pa cesto daje nerealne procene
print("Prosecna zarada: ", data["ConvertedCompYearly"].mean())

# mozemo da primenimo i agregatne funkcije na citav skup podataka
# medjutim, moramo da se ogranicimo samo na numericke vrednosti
# (pitanje se postavlja sta bi bila medijana stringova?)
print(data.median(numeric_only=True))
# agregatne funckije mozemo da primenjujemo i po redovima
# (ovde to nema nikakvog smisla, ali cisto radi ilustracije)
print(data.median(axis=1, numeric_only=True))

# ako nam treba brzi statisticki pregled numerickih podataka mozemo da iskoristimo
# metod describe()
# describe() za svaku numericku kolonu prikazuje min, max, median, count, std
# i kvantile (25%, 50%, 75%)
# count predstavlja broj redova koji nisu nedostajuci, tj. nisu NaN
print(data.describe())

# brzi opis mozemo da primenimo i na konkretne kolone
print(data["ConvertedCompYearly"].describe())

# broj ucesnika ankete koji su dali odgovor
print("Broj ispitanika: ", data["ConvertedCompYearly"].size)
print("Broj onih koji su odgovorili: ", data["ConvertedCompYearly"].count())
print("Procenat koji je ucestvova: ", data["ConvertedCompYearly"].count()/data["ConvertedCompYearly"].size)
# bitno: count() prebrojava samo koliko ima redova koji nisu nedostajuci, tj. imaju vrednost i nisu NaN
#        count() ne prebrojava koliko ima razlicitih odgovora

# metod value_count() vraca broj razlictih odgovora i koliko ljudi je kako odgovorilo
# mozemo da prebrojimo koliko ima razlicitih industrija koje se javljaju u anketi kao i broj
# ljudi koji rade u tim indrustrijama
print(data["Industry"].head(15))
print("Broj razlictih industrija primene: ")
# value_counts() ne uzima u obzir nedostajuce odgovore
print(data["Industry"].value_counts())

# na primer, zanima nas popularnost razlictih modela rada
print("Modeli rada: ")
# ako zelimo da saznamo orginalno pitanje, treba da pogledato data_schema skup podataka
# indeksiranje je moguce po imenu kolone, jer smo indeks postavli na qname
print(data_schema.loc["RemoteWork"])
# ako nas ne zanimaju sirovi brojevi, vec procenti, kao argument mozemo da prosledimo
# normalize=True
print(data["RemoteWork"].value_counts(normalize=True))

# medjutim, ovako posmatrani podaci ne daju bas neke posebne informacije o 
# odgovorima, mozda bi bilo bolje da posmatramo podatke po zemljama koje se 
# javljaju u anketi

# mozda bi bilo zanimljivo da pogledamo ove statistike prema zemljama ili necemu drugom
# da bismo to postigli, potrebno je da prvo podatke grupisemo prema zemljama
# za to se koristi group_by() metod

# pre bilo cega pogledacemo brojeve ispitanika prema zemljama
print(data["Country"].value_counts())

# sada zelimo da primenimo group_by()
# prvo moramo da podelimo originalni skup prema zemljama
# kao rezultat dobijamo vise odvojenih skupova podataka 
# koji su grupisani po zemljama
country_groups = data.groupby(["Country"])
# grupe, tj. skupove podataka za svaku zemlju mozemo da indeksiramo prema imenima zemalja
# ovaj rezultat je slican kao da smo primenili filter po zemlji na originalni skup podataka
print(country_groups.get_group("India"))

# medjutim, morao bih da primenjujem onoliko filtera koliko ima zemalja u skupu podataka
# filteri imaju smisla kada nas zanima rezultat samo za jednu konkretnu zemlju. ako nam trebaju
# gripusane informacije o vise zemalja, tada filteri nemaju smisla i treba da koristimo groupby()
filt = data["Country"] == "India"
# mozemo da odredimo popularnost oblika rada u Indiji
print("Popularnost oblika rada u Indiji: ")
print(data.loc[filt]["RemoteWork"].value_counts(normalize=True))

# sada kada imamo podatke grupisane po zemljama, mozemo da odredimo na primer popularnost
# oblika rada u svakoj zemlji
print("Popularnost oblika rada po zemljama: ")
print(country_groups["RemoteWork"].value_counts(normalize=True).head(50))

# na primer, mozemo da vidimo kakva je situacija u Srbiji
print("Popularnost oblika rada u Srbiji: ")
print(country_groups.get_group("Serbia")["RemoteWork"].value_counts(normalize=True))
# drugi nacin koji je moguc, jer kreirani data frame ima visestruke indekse
# jedan indeks je zemlja, a drugi indeks ce biti "RemoteWork"
print(country_groups["RemoteWork"].value_counts(normalize=True).loc["Serbia"])

# medijalna zarada po zemljama
print("Medijalne zarade po zemljama: ")
print(country_groups["ConvertedCompYearly"].median())
print("Medijalna zarada u Srbiji: ")
print(country_groups["ConvertedCompYearly"].median().loc["Serbia"])

# sta ako su nam potrebne visetruke agregatne funkcije koje zelimo istovremeno da 
# primenimo na nase grupisane podatke
# za tako nesto koristmo agg() metod i kao argument prosledjujemo nazive svih 
# agregatnih fukcija koje zelimo da primenimo
print("Statistike zarada po zemljama: ")
print(country_groups["ConvertedCompYearly"].agg(['median', 'mean', 'max', 'min']))
print("Statistike zarada u Srbiji: ")
print(country_groups["ConvertedCompYearly"].agg(['median', 'mean', 'max', 'min']).loc["Serbia"])

# na primer, sta ako treba da saznamo koliko ljudi u kojoj zemlji zna da koristi Pajton

# prvo mozemo da probamo to da odredimo na ranije poznati nacin, tj. uz pomoc filtera
# za konkretnu zemlju
filt = data["Country"] == "India"
# ono koji nisu odgovorili na pitanje, tj. polja gde je vrednost NaN, obelezicemo kao false
# na kraju sabracemo sve koji znaju pajton sa metodom sum()
print(data.loc[filt]["LanguageHaveWorkedWith"].str.contains('Python', na=False).sum())

# do sada, kada smo hteli da prosirimo ideju iz filtera na groupby() objekat to je islo
# pravolinijski
# problem je sto ne mozemo da koristimo metod str na groupby() objektu direktno
# vec moramo da primenimo str na svaki series objekat od kojis je sacinjen groupby() objekat
# country_groups["LanguageHaveWorkedWith"].str.contains("Python").sum()

# potrebno je da prvo primenimo apply metod na sve series objekte u grupi
# pisemo anonimnu funkciju koja treba da se primeni na svaki series objekat 
# dakle x je pojedinicni series objekat cije sve elemente zelimo prvo da prebacimo u string
# zatim da obelezimo sve one redove u seriji koji sadrze rec "Python" i onda sve tacke da saberemo
# dakle, rezultat za svaku grupu, tj. zemlju je jedan broj koji govori koliko anketiranih zna pajton
print(country_groups["LanguageHaveWorkedWith"].apply(lambda x: x.str.contains("Python", na=False).sum()))

# kao i do sada sirove brojke nisu bas informativne, mozda je bolje da izracunamo procente
# koji odgovaraju broju anketiranih koji znaju pajton po zemljama
print("Procenat ljudi koji znaju Pajton po zemljama:")
# delimo sa len(x), ne sa x.count(), jer x.count() ne broji one koji nisu odgovorili na pitanje,
# a oni svakako ucestvuju u ukupnom broju ispitanika
print(country_groups["LanguageHaveWorkedWith"].apply(lambda x: x.str.contains("Python", na=False).sum()/len(x)))


# drugi nacin
country_respondents = data["Country"].value_counts()
country_uses_python = country_groups["LanguageHaveWorkedWith"].apply(lambda x: x.str.contains("Python", na=False).sum())
# kao rezultat ove dve naredbe smo dobili dve series promenljive koje treba da kombinujemo
# tj. nadovezacemo ih jednu na drugu
# nadovezujemo ih po kolona, jer treba da nam se poklope indeksi, tj. zemlje
# sortiracemo ih po indeksu
data_python = pd.concat([country_respondents, country_uses_python], axis="columns", sort=True)
# preimenovacemo kolone da oslikavaju ono sto predstavljaju
data_python.rename(columns={"count": "NumRespondents", "LanguageHaveWorkedWith": "NumKnowsPython"}, inplace=True)
# sada cemo da napravimo novu kolonu i da definisemo kako je izracunavamo
data_python["PctKnowsPython"] = data_python["NumKnowsPython"]/data_python["NumRespondents"]
print("Procenat ljudi koji znaju Pajton po zemljama:")
print(data_python)

# sortiramo zemlje prema najvisem procentu
data_python.sort_values(by="PctKnowsPython", ascending=False, inplace=True)
print("Procenat ljudi koji znaju Pajton po zemljama (sortirano opadajuce):")
print(data_python.head(50))
# sada mozemo da proverimo i kakva je situacija u Srbiji
print("Situacija u Srbiji: ")
print(data_python.loc["Serbia"])